CON
   ' First, define the repeat counts for various clock counts.
   repeatDelay_320clks  =  0 ' verified
   repeatDelay_544clks  =  1 ' verified
   repeatDelay_816clks  =  2 ' 48 additional clocks?
   repeatDelay_1040clks =  3
   repeatDelay_1264clks =  4
   repeatDelay_1456clks =  5
   repeatDelay_1680clks =  6
   repeatDelay_1936clks =  7
   repeatDelay_2160clks =  8
   repeatDelay_2352clks =  9
   repeatDelay_2576clks = 10
   repeatDelay_2800clks = 11
   repeatDelay_3024clks = 12
   repeatDelay_3248clks = 13
   repeatDelay_3472clks = 14
   repeatDelay_3728clks = 15
   repeatDelay_3952clks = 16
   repeatDelay_4144clks = 17
   ' next would be 4368 clocks
   ' the clock time for usDelayShort(0) is 4304 clocks, so we
   ' can stop here.


   ' These constants are used with the "repeat" statement with
   ' an empty body to produce a short delay. If we have:
   ' OBJ
   '    DLY : "Delay"
   '
   ' Then we produce a short delay like this:
   '    a := foo
   '    repeat DLY#repeatDelay112_9p1us ' min delay 9.1 us at 112 MHz, 12.8 us at 80Mhz
   '      ' empty loop body
   '    b := bar

{
   ' 80MHz (8 * 10 MHz, 16 * 5 MHz)
   repeatDelay80_4us     = repeatDelay_320clks
   repeatDelay80_6p8us   = repeatDelay_544clks
   repeatDelay80_10us    = repeatDelay_800clks
   repeatDelay80_12p8us  = repeatDelay_1024clks
   ' each additional one increases by 2.8 us
   ' continue to the min usDelay for usDelayShort()

   ' 96 MHz (16 * 6 MHz)
   repeatDelay96_3p3us   = repeatDelay_320clks
   repeatDelay96_5p7us   = repeatDelay_544clks
   repeatDelay96_8p3us   = repeatDelay_800clks
   repeatDelay96_10p7us  = repeatDelay_1024clks
   ' each additional one increases by about 2.33 us
   ' continue to the min usDelay for usDelayShort()

   ' 100 MHz (16 * 6.25 MHz)
   repeatDelay100_3p2us  = repeatDelay_320clks
   repeatDelay100_5p4us  = repeatDelay_544clks
   repeatDelay100_8us    = repeatDelay_800clks
   repeatDelay100_10p2us = repeatDelay_1024clks
   ' each additional one increases by 2.44 us

   ' 112 MHz (16 * 7 Mhz)
   repeatDelay112_2p9us  = repeatDelay_320clks
   repeatDelay112_4p9us  = repeatDelay_544clks
   repeatDelay112_7p1us  = repeatDelay_800clks
   repeatDelay112_9p1us  = repeatDelay_1024clks
   ' each additional one increases by about 2.18 us

   ' 120 MHz (16 * 7.5 Mhz)
   repeatDelay120_2p7us  = repeatDelay_320clks
   repeatDelay120_4p5us  = repeatDelay_544clks
   repeatDelay120_6p7us  = repeatDelay_800clks
   repeatDelay120_8p5us  = repeatDelay_1024clks
   ' each additional one increases by about 1.87 us
}


  _1us  = 1_000_000                                     ' divisor for 1 us  
  _10us = 1_000_000 /        10                         ' Divisor for 10 us
  _1ms  = 1_000_000 /     1_000                         ' Divisor for 1 ms
  _1s   = 1_000_000 / 1_000_000                         ' Divisor for 1 s

  ClocksForEmptyCall = 672

{
  '                      Fake      Real
  ClocksForNoDelay     =           1520
  WaitcntOverhead      =           3027-398
  WaitcntOverhead2     =           2339-398
}

VAR

'  long delay
'  long syncpoint
'  long clkcycles

' "repeat n" takes n*224 + 352 clocks, or n*2.8 + 4.4 us at 80MHz

PUB emptyDelay
  ' An empty subroutine
  ' A call to this subroutine takes 672 clocks, including the call
  '   overhead.
  ' At an 80MHz clock, this is 672/80 us, or 8.4  us.
  ' At a  96MHz clock, this is 672/96 us, or 7    us.
  ' At a 100MHz clock, this is               6.72 us.
  ' 
  ' So any timing _call_ cannot be shorter than this.

    
PUB usDelayShort( usec )
  '' Delay for the specified number of microseconds.
  '' Note that there will be a minimum delay possible, at
  '' about 43 us at 100Mhz, or 54 us at 80MHz.

  waitcnt( ((((CLKFREQ / _1us) * usec) - 3935) #> 381) + cnt )
  ' min clocks: 4304, or 53.8 us at 80 MHz, 43.04 us at 100 MHz
  ' 3935 is the "magic number" that makes the timings line up,
  '   including the overhead for the call and return, and calculations.

PUB usDelay( us ) | t
  ' The minimum number of clocks seems to be about 5120, or
  ' 64 us
  
  if ( us => 2_000_000 )
    ' gross waits of one second, until us is "small enough"
    ' adjust init_overhead for long delays
    t := cnt - 1136{init_overhead}
    repeat while ( us => 2_000_000 )
      us -= 1_000_000 ' one less second
      waitcnt( t += CLKFREQ ) ' wait until next second
                              ' no need to subtract overhead, since we are advancing
                              ' to the exact ms "fenceposts"
  ' Now, perform the final wait
  waitcnt( ((((CLKFREQ / 1_000_000) * us) - 4724) #> 381) + cnt )
 

PUB msDelay( ms ) | t
'' delay ms milliseconds
'' ms > 0, up to 2^31 - 1 == 2_147_483_647 ms ~ 2_147_483.647 s ~ 35_791.4 m ~ 596.5 h
''                        ~ 24.86 days

  ' Adjust 'function_overhead' so that the timings are correct.
  t := cnt{loop_overhead}
  repeat while ( ms => 2_000 ) ' any reasonable number of seconds works.
    ms -= 1_000 ' one less second
    waitcnt( t += CLKFREQ ) ' wait until next second
                            ' no need to subtract overhead, since we are advancing
                            ' to the exact ms "fenceposts"

  ' We have reduced this down to under 2 seconds, but large enough
  ' that we can subtract the offset without making the count too small
  waitcnt( ((((CLKFREQ / 1_000) * ms) - 5044{func_overhead}) #> 381) + cnt )


PUB sDelay( s ) | t
'' delay s seconds
'' s > 0, up to 2^31 - 1 == 2_147_483_647 s ~ 35_791_394 m ~ 596_523.25 h
''                        ~ 24_855 days, about 68 years.

  ' Adjust 'function_overhead' so that the timings are correct.
  t := cnt - 1172{function_overhead}
  repeat s
    waitcnt( t += CLKFREQ ) ' wait until next second



{
PUB clocksDelay( clks )
'' Delay the specified number of clocks
  if ( clks > ClocksForNoDelay )
    waitcnt( ( (clks - WaitcntOverhead) #> 381 ) + cnt )
  ' If we do not do the waitcnt,
  '      this takes 1520 clocks, or 19 us at 80Mhz
  ' If we do the waitcnt,
  '      this takes 3024 clock minimum, or 37.8 us min at 80Mhz
 
PUB clocksDelay2( clks )
'' Delay the specified number of clocks

  waitcnt( ( (clks - WaitcntOverhead2) #> 381 ) + cnt )
           ' This takes a minimum of 2336 clocks, or 29.2 us at 80Mhz
  ' Because SPIN is read from the hub, this will only work
  ' for 16-clock boundaries!
  ' 2336, 2352, 2366, etc.
  ' So, we have to adjust the overhead, so that the delay is
  ' as long or longer than then desired clock count.
  ' So, we map 2337 to 2352, or to 382 + 16:
  '    2337 - WCO == 398.
  '    2337 - 398 == WCO = 1939
  '    2352 - 1939 ==     413
}
      